@@ -1,12 +1,13 @@
-from .model import *
 from .krige import *
 from .mesh import *
-from .gp import Observation
+from .model import *
+from .mean import *
 
-__version__ = '0.7.2'
+__version__ = '0.8.0'
 
 __all__ = []
-__all__.extend(model.__all__)
-__all__.extend(krige.__all__)
+__all__.extend(mean.__all__)
 __all__.extend(mesh.__all__)
-__all__.extend('Observation')
+__all__.extend(model.__all__)
+__all__.extend(kernel.__all__)
+__all__.extend(krige.__all__)
