@@ -14,20 +14,46 @@ with warnings.catch_warnings():
     import tensorflow as tf
     import tensorflow_probability as tfp
     tfd = tfp.distributions
-    from tensorflow.linalg import LinearOperatorFullMatrix as LOFullMatrix
-    from tensorflow.linalg import LinearOperatorBlockDiag as LOBlockDiag
 
 import tensorflow_probability as tfp
 
-from . import gp
-from .op import Op
+from . import mean as mn
+from . import kernel as krn
+from .op import Op, SingletonTraceType
 from .param import PaperParameter, ParameterSpace, Bound
 from .metric import Euclidean, PerAxisDist2
 from .param import get_parameter_values, ppp, upp, bpp
 
 MVN = tfp.distributions.MultivariateNormalTriL
 
-__all__ = ['Model', 'Featurizer', 'NormalizingFeaturizer']
+__all__ = ['Featurizer', 'GP', 'Mix', 'Model', 'NormalizingFeaturizer']
+
+@dataclass
+class GP:
+    mean: mn.Trend = None
+    kernel: krn.Kernel = None
+
+    def __post_init__(self):
+        if self.mean is None or self.mean == 0:
+            self.mean = mn.ZeroTrend()
+        assert self.kernel is not None
+
+    def __add__(self, other):
+        return GP(self.mean + other.mean, self.kernel + other.kernel)
+
+    def __tf_tracing_type__(self, context):
+        return SingletonTraceType(self)
+
+    def gather_vars(self):
+        return set(self.mean.gather_vars()) | set(self.kernel.gather_vars())
+
+    def reg(self, sp):
+        return self.kernel.reg(sp)
+
+def Mix(inputs, weights=None):
+    return GP(
+        mn.Mix([i.mean for i in inputs], weights), 
+        krn.Mix([i.kernel for i in inputs], weights))
 
 class NormalizingFeaturizer:
     """
@@ -72,16 +98,12 @@ class Featurizer:
 def e(x, a=-1):
     return tf.expand_dims(x, a)
 
-def block_diag(blocks):
-    """Return a dense block-diagonal matrix."""
-    return LOBlockDiag([LOFullMatrix(b) for b in blocks]).to_dense()
+@tf.function
+def gp_covariance(gp, locs, cats, p):
+    return gp_covariance2(gp, locs, cats, locs, cats, 0, p)
 
 @tf.function
-def gp_covariance(covariance, observation, locs, cats, p):
-    return gp_covariance2(covariance, observation, locs, cats, locs, cats, 0, p)
-
-@tf.function
-def gp_covariance2(covariance, observation, locs1, cats1, locs2, cats2, offset, p):
+def gp_covariance2(gp, locs1, cats1, locs2, cats2, offset, p):
     """
     `offset` is i2-i1, where i1 and i2 are the starting indices of locs1
     and locs2.  It is used to create the diagonal non-zero elements
@@ -99,55 +121,15 @@ def gp_covariance2(covariance, observation, locs1, cats1, locs2, cats2, offset,
     cache['offset'] = offset
     cache['locs1'] = locs1
     cache['locs2'] = locs2
+    cache['cats1'] = cats1
+    cache['cats2'] = cats2
     cache['per_axis_dist2'] = PerAxisDist2().run(cache, p)
     cache['euclidean'] = Euclidean().run(cache, p)
 
-    MM, CC = zip(*[c.run(cache, p) for c in covariance])
-    M = tf.stack(MM, axis=-1) # [locs, hidden].
-    C = tf.stack(CC, axis=-1) # [locs, locs, hidden].
-
-    numobs = len(observation)
-
-    if numobs == 0:
-        assert len(covariance) == 1, 'With no observation model, I only want one covariance model'
-        C = tf.cast(C[..., 0], tf.float64)
-        M = tf.cast(M[..., 0], tf.float64)
-        return M, C
-
-    A = tf.convert_to_tensor(gp.get_parameter_values([o.coefs for o in observation], p)) # [surface, hidden].
-    M = tf.gather(tf.einsum('lh,sh->ls', M, A), cats1, batch_dims=1) # [locs]
-
-    Aaug1 = tf.gather(A, cats1) # [locs, hidden].
-    Aaug2 = tf.gather(A, cats2) # [locs, hidden].
-    outer = tf.einsum('ac,bc->abc', Aaug1, Aaug2) # [locs, locs, hidden].
-    C = tf.einsum('abc,abc->ab', C, outer) # [locs, locs].
-
-    catcounts1 = tf.math.bincount(cats1, minlength=numobs, maxlength=numobs)
-    catcounts2 = tf.math.bincount(cats2, minlength=numobs, maxlength=numobs)
-    catindices1 = tf.math.cumsum(catcounts1, exclusive=True)
-    catindices2 = tf.math.cumsum(catcounts2, exclusive=True)
-    catdiffs = tf.unstack(catindices2 - catindices1, num=numobs)
-    locsegs1 = tf.split(locs1, catcounts1, num=numobs)
-    locsegs2 = tf.split(locs2, catcounts2, num=numobs)
-
-    CC = [] # Observation noise submatrices.
-    MM = [] # Mean subvectors.
-    for sublocs1, sublocs2, catdiff, o in zip(locsegs1, locsegs2, catdiffs, observation):
-        cache['offset'] = offset + catdiff
-        cache['locs1'] = sublocs1
-        cache['locs2'] = sublocs2
-        cache['per_axis_dist2'] = PerAxisDist2().run(cache, p)
-        cache['euclidean'] = Euclidean().run(cache, p)
-        Msub, Csub = o.noise.run(cache, p)
-        CC.append(Csub)
-        MM.append(Msub)
-
-    C += block_diag(CC)
-    C = tf.cast(C, tf.float64)
-
-    M += tf.concat(MM, axis=0)
+    M = gp.mean.run(cache, p)
+    C = gp.kernel.run(cache, p)
     M = tf.cast(M, tf.float64)
-
+    C = tf.cast(C, tf.float64)
     return M, C
 
 @tf.function
@@ -159,19 +141,19 @@ def mvn_log_pdf(u, m, cov):
     return tf.cast(-0.5 * (logdet + quad), tf.float32)
 
 @tf.function
-def gp_log_likelihood(data, surf_params, covariance, observation):
-    m, S = gp_covariance(covariance, observation, data['locs'], data['cats'], surf_params)
+def gp_log_likelihood(data, surf_params, gp):
+    m, S = gp_covariance(gp, data['locs'], data['cats'], surf_params)
     u = tf.cast(data['vals'], tf.float64)
     return mvn_log_pdf(u, m, S)
 
-def gp_train_step(optimizer, data, parameters, parameter_space, covariance, observation, reg=None):
+def gp_train_step(optimizer, data, parameters, parameter_space, gp, reg=None):
     with tf.GradientTape() as tape:
         sp = parameter_space.get_surface(parameters)
 
-        ll = gp_log_likelihood(data, sp, covariance, observation)
+        ll = gp_log_likelihood(data, sp, gp)
 
         if reg:
-            reg_penalty = reg * tf.reduce_sum([c.reg(sp) for c in covariance])
+            reg_penalty = reg * gp.reg(sp)
         else:
             reg_penalty = 0.
 
@@ -197,8 +179,7 @@ def check_parameters(pps: List[PaperParameter], values: Dict[str, float]) -> Dic
 
 @dataclass
 class Model():
-    latent: List[gp.GP] = None
-    observed: List[gp.Observation] = None
+    gp: GP
     parameters: Dict[str, np.ndarray] = None
     parameter_sample_size: Optional[int] = None
     locs: np.ndarray = None
@@ -222,7 +203,7 @@ class Model():
                                 return x1[:, 0], x1[:, 1], x1[:, 0]**2, x1[:, 1]**2.
                     Default is None.
 
-                latent : List[gp.GP]
+                latent : List[GP]
                      Name of the covariance function to use in the GP.
                      Should be 'squared-exp' or 'gamma-exp'.
                      Default is 'squared-exp'.
@@ -239,14 +220,6 @@ class Model():
         Performs Gaussian process training and prediction.
         '''
 
-        super().__init__()
-
-        assert self.latent is not None, 'I need at least one latent variable'
-        if isinstance(self.latent, gp.GP):
-            self.latent = [self.latent]
-
-        if self.observed is None: self.observed = []
-
         # Default reporting function.
         def default_report(p, prefix=None):
             if prefix: print(prefix, end=' ')
@@ -273,10 +246,8 @@ class Model():
 
         # Collect paraameters.
         if self.parameters is None: self.parameters = {}
-        vv = {v for c in self.latent for v in c.gather_vars()}
-        vv |= {v for o in self.observed for v in o.gather_vars()}
 
-        self.parameter_space = ParameterSpace(check_parameters(vv, self.parameters))
+        self.parameter_space = ParameterSpace(check_parameters(self.gp.gather_vars(), self.parameters))
 
     def fit(self, locs, vals, cats=None,
         step_size=0.01, iters=100, reg=None):
@@ -302,7 +273,7 @@ class Model():
             t0 = time.time()
             while j < (i + 1) * iters / 10:
                 p, ll, reg_penalty = gp_train_step(optimizer, self.data, up, self.parameter_space,
-                    self.latent, self.observed, reg)
+                    self.gp, reg)
                 j += 1
 
             time_elapsed = time.time() - t0
@@ -343,7 +314,7 @@ class Model():
         # Unnormalized log posterior distribution.
         def g(up):
             sp = self.parameter_space.get_surface(up)
-            return gp_log_likelihood(self.data, sp, self.latent, self.observed)
+            return gp_log_likelihood(self.data, sp, self.gp)
 
         def f(*up_flat):
             up = tf.nest.pack_sequence_as(initial_up, up_flat)
@@ -458,8 +429,7 @@ class Model():
         p = self.parameter_space.get_surface(up)
 
         m, S = gp_covariance(
-            self.latent,
-            self.observed,
+            self.gp,
             tf.constant(locs, dtype=tf.float32),
             None if cats is None else tf.constant(cats, dtype=tf.int32),
             p)
@@ -504,8 +474,7 @@ class Model():
                 locs2, cats2 = locs2[perm], cats2[perm]
 
             _, A12 = gp_covariance2(
-                self.latent,
-                self.observed,
+                self.gp,
                 tf.constant(locs1, dtype=tf.float32),
                 None if cats1 is None else tf.constant(cats1, dtype=tf.int32),
                 tf.constant(locs2, dtype=tf.float32),
@@ -514,8 +483,7 @@ class Model():
                 parameters)
 
             m2, A22 = gp_covariance(
-                self.latent,
-                self.observed,
+                self.gp,
                 tf.constant(locs2, dtype=tf.float32),
                 None if cats2 is None else tf.constant(cats2, dtype=tf.int32),
                 parameters)
@@ -554,8 +522,7 @@ class Model():
                 locs1, vals1, cats1 = locs1[perm], vals1[perm], cats1[perm]
 
             m1, A11 = gp_covariance(
-                self.latent,
-                self.observed,
+                self.gp,
                 tf.constant(locs1, dtype=tf.float32),
                 None if cats1 is None else tf.constant(cats1, dtype=tf.int32),
                 parameters)
